@@ -1,6 +1,6 @@
-shinyServer(function(input, output) {
+shinyServer(function(input, output, session) {
   output$currentTime <- renderText({
-    invalidateLater(1000)
+    invalidateLater(1000, session)
     paste("The current time is", Sys.time())
   })
-})
\ No newline at end of file
+})
